namespace CustomLocalization.Web
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Configuration;
    using System.Data;
    using System.Data.Common;
    using System.Data.SqlClient;
    using System.Globalization;

    /// <summary>
    /// This class provides the Data Access to the database resources
    /// </summary>
    public class DbResourceDataManager
    {
        /// <summary>
        /// Default constructor. 
        /// </summary>
        public DbResourceDataManager()
        {
        }

        /// <summary>
        /// Sql Connection String
        /// </summary>
        public String ConnectionString
        {
            get
            {
                return ConfigurationManager.ConnectionStrings["acmeConnectionString"].ConnectionString;
            }
        }

        /// <summary>
        /// Returns a fully normalized list of resources that contains the most specific
        /// locale version for the culture provided.
        ///                 
        /// This means that this routine walks the resource hierarchy and returns
        /// the most specific value in this order: de-ch, de, invariant.
        /// </summary>
        /// <param name="CultureName"></param>
        /// <param name="ResourceSet"></param>
        /// <returns></returns>
        public Dictionary<string, string> GetResourceSetNormalizedForLocaleId(string CultureName, string ResourceSet)
        {
            if (CultureName == null)
                CultureName = "";

            Dictionary<string, string> resDictionary = new Dictionary<string, string>();

            SqlConnection conn = null;
            SqlCommand cmd = null;
            SqlDataReader reader = null;

            try 
            {
                conn = new SqlConnection(ConnectionString);
                conn.Open();

                string sql =
                "select R.[Key], R.LocaleId, R.Value" +
                " from StringResource R JOIN StringResourceTag RT on RT.StringResourceId = R.Id JOIN Tag T on T.Id = RT.TagId" +
                " where T.Description=@ResourceSet and (LocaleId = '' {0} ) order by R.[Key], R.LocaleId DESC";

                cmd = new SqlCommand();
                cmd.CommandType = CommandType.Text;
                cmd.Connection = conn;

                // use like parameter or '' if culture is empty/invariant
                string localeFilter = "";

                cmd.Parameters.AddWithValue("@ResourceSet", ResourceSet);

                if (!string.IsNullOrEmpty(CultureName))
                {
                    localeFilter += " OR LocaleId = @LocaleId";
                    cmd.Parameters.AddWithValue("@LocaleId", CultureName);

                    // *** grab shorter version
                    if (CultureName.Contains("-"))
                    {
                        localeFilter += " OR LocaleId = @LocaleId1";
                        cmd.Parameters.AddWithValue("@LocaleId1", CultureName.Split('-')[0]);
                    }
                }

                sql = string.Format(sql, localeFilter);
                cmd.CommandText = sql;

                // Execute the query
                reader = cmd.ExecuteReader();

                if (reader != null)
                {
                    string lastResourceKey = "xxxyyy";

                    while (reader.Read())
                    {
                        // only pick up the first ID returned - the most specific locale
                        string resourceKey = reader["Key"].ToString();
                        if (resourceKey == lastResourceKey)
                            continue;
                        lastResourceKey = resourceKey;

                        // Read the value into this
                        string resourceValue = null;
                        resourceValue = reader["Value"] as string;

                        if (resourceValue == null)
                        {
                            resourceValue = "";
                        }

                        resDictionary.Add(resourceKey, resourceValue);
                    }
                }
            }
            catch { }
            finally
            {
                // close reader and connection
                if (reader != null)
                {
                    reader.Close();
                }

                if (conn!=null) 
                {
                    conn.Close();
                }
            }

            return resDictionary;
        }
    }
}
