﻿using System;
using System.Configuration;
using System.Globalization;
using System.IO;
using System.Web;
using System.Xml;

namespace CustomLocalization.Web {
    public class ResxHandler : IHttpHandler
    {
        private CultureInfo Culture;
        private HttpContext Context;
        private string ResourceSet;

        public bool IsReusable { get { return false; } }

        public const string BASERESOURCEPATH = "\\Resources\\";

        private static string resourceBasePath = null;

        public static string ResourceBasePath
        {
               get
               {
                   if (resourceBasePath == null)
                   {
                       resourceBasePath = ConfigurationManager.AppSettings["ResourceBasePath"];
                       if (string.IsNullOrEmpty(resourceBasePath))
                       {
                           resourceBasePath = BASERESOURCEPATH;
                       }

                       if (resourceBasePath != null && !resourceBasePath.EndsWith("\\"))
                       {
                           resourceBasePath += "\\";
                       }
                   }

                   return resourceBasePath;
               }
        }

        private static bool resourceCacheDurationSet = false;

        private static TimeSpan resourceCacheDuration;

        public static TimeSpan ResourceCacheDuration
        {
            get
            {
                if (!resourceCacheDurationSet)
                {
                    resourceCacheDurationSet = true;

                    // Set to 12 hours
                    resourceCacheDuration = new TimeSpan(12, 0, 0);

                    string strDuration = ConfigurationManager.AppSettings["ResourceCacheDuration"];
                    if (!string.IsNullOrEmpty(strDuration))
                    {
                        int duration;
                        if (int.TryParse(strDuration, out duration))
                        {
                            resourceCacheDuration = new TimeSpan(duration, 0, 0);
                        }
                    }
                }

                return resourceCacheDuration;
            }
        }

        public void ProcessRequest(HttpContext context)
        {
            Context = context;
            Culture = System.Threading.Thread.CurrentThread.CurrentCulture;

            string culture = context.Request.Params["c"];

            ResourceSet = context.Request.Params["rs"];

            FileInfo resxFileInfo = new FileInfo(Context.Server.MapPath(Context.Request.Path));

            string resxFileName = resxFileInfo.Name;
            string[] fileParts = resxFileName.Split('.');
            if (fileParts.Length > 2)
            {
                culture = fileParts[1];
            }

            if (fileParts.Length > 1)
            {
                ResourceSet = fileParts[0];
            }

            if (!string.IsNullOrEmpty(culture)) {
                try
                {
                    Culture = new CultureInfo(culture);
                }
                catch (Exception ex)
                {
                    string errMsg = ex.Message;

                    // throw an error
                    Utilities.SendResponse(Context, 500,
                                           "The string " + culture + " is not recognized as a valid culture.");
                    return;
                }
            }

            // Get the RESX file for the locale
            string resxFile = XapHandler.GetResourceFilePath(Context, ResourceSet, Culture.Name);

            if (string.IsNullOrEmpty(resxFile) || !File.Exists(resxFile))
            {
                Utilities.SendResponse(Context, 500, "The Resx file does not exist.");
                return;
            }
            else
            {
                Utilities.TransmitFile(context, resxFile, "text/xml", false);
            }
        }

    }
}