﻿using System;
using System.IO;
using System.Web;
using Ionic.Zip;

namespace CustomLocalization.Web
{
    public static class Utilities
    {
        public static void SendResponse(HttpContext context, int statusCode, string message)
        {
            if (context == null) return;

            context.Response.StatusCode = statusCode;
            if(!string.IsNullOrEmpty(message))
            {
                context.Response.StatusDescription = message;
            }

            context.Response.End();
        }

        public static void AddFileToZip(string zipFile, string fileToAdd, string directoryPathInZip)
        {
            if (string.IsNullOrEmpty(zipFile) || string.IsNullOrEmpty(fileToAdd))
                return;

            using (ZipFile zip = ZipFile.Read(zipFile))
            {
                zip.AddFile(fileToAdd, directoryPathInZip);
                zip.Save();
            }
        }

        public static void TransmitFile(HttpContext context, string filePath, string contentType, bool deleteFile)
        {
            if (context == null) return;
            if (string.IsNullOrEmpty(filePath)) return;

            FileInfo file = new FileInfo(filePath);

            try
            {
                if (file.Exists)
                {
                    context.Response.AppendHeader("Content-Length", file.Length.ToString());
                    context.Response.ContentType = contentType;

                    if (!context.IsDebuggingEnabled)
                    {
                        context.Response.Cache.SetCacheability(HttpCacheability.Public);
                        context.Response.ExpiresAbsolute = DateTime.UtcNow.AddDays(1);
                        context.Response.Cache.SetLastModified(DateTime.UtcNow);
                    }

                    context.Response.TransmitFile(file.FullName);

                    if (context.Response.IsClientConnected)
                    {
                        context.Response.Flush();
                    }
                }
                else
                {
                    Utilities.SendResponse(context, 404, "File Not Found (" + filePath + ").");
                }
            }
            finally
            {
                if (deleteFile && file.Exists)
                {
                    file.Delete();
                }
            }
        }

        public static string GetAppRelativePath(string logicalPath)
        {
            logicalPath = logicalPath.ToLower();

            string appPath = string.Empty;

            if (HttpContext.Current != null)
            {
                appPath = HttpContext.Current.Request.ApplicationPath.ToLower();
                if (appPath != "/")
                    appPath += "/";
                else
                    return logicalPath.TrimStart('/');
            }
            else
            {
                return logicalPath.TrimStart('/');
            }

            return logicalPath.Replace(appPath, "");
        }
    }
}